#pragma once

#include <adapta.h>

G_BEGIN_DECLS

#define ADAP_TYPE_DEMO_PAGE_LISTS (adap_demo_page_lists_get_type())

G_DECLARE_FINAL_TYPE (AdapDemoPageLists, adap_demo_page_lists, ADAP, DEMO_PAGE_LISTS, AdapBin)

G_END_DECLS
